@@ -1,28 +1,25 @@
-#' @title Read/Export records from a REDCap project --still in development
+#' @title
+#' Read/Export records from a REDCap project --still in development
 #'
-#' @description This function uses REDCap's API to select and return data.
+#' @description
+#' This function uses REDCap's API to select and return data.
 #' This function is still in development.
 #'
-#' @param redcap_uri The URI (uniform resource identifier) of the REDCap
-#' project.  Required.
+#' @param redcap_uri The
+#' [uri](https://en.wikipedia.org/wiki/Uniform_Resource_Identifier)/url
+#' of the REDCap server
+#' typically formatted as "https://server.org/apps/redcap/api/".
+#' Required.
 #' @param token The user-specific string that serves as the password for a
 #' project.  Required.
 #' @param records An array, where each element corresponds to the ID of a
 #' desired record.  Optional.
-#' @param records_collapsed A single string, where the desired ID values
-#' are separated by commas.  Optional.
 #' @param fields An array, where each element corresponds to a desired
 #' project field.  Optional.
-#' @param fields_collapsed A single string, where the desired field names
-#' are separated by commas.  Optional.
 #' @param forms An array, where each element corresponds to a desired project
 #' field.  Optional.
-#' @param forms_collapsed A single string, where the desired form names are
-#' separated by commas.  Optional.
 #' @param events An array, where each element corresponds to a desired project
 #' event.  Optional.
-#' @param events_collapsed A single string, where the desired event names are
-#' separated by commas.  Optional.
 #' @param raw_or_label A string (either `'raw'` or `'label'` that specifies
 #' whether to export the raw coded values or the labels for the options of
 #' multiple choice fields.  Default is `'raw'`.
@@ -52,16 +49,32 @@
 #' (https://stat.ethz.ch/R-manual/R-devel/library/base/html/as.POSIXlt.html)
 #' value.
 #' If not specified, REDCap will assume no end time.
+#' @param blank_for_gray_form_status A boolean value that specifies whether
+#' or not to export blank values for instrument complete status fields that have
+#' a gray status icon. All instrument complete status fields having a gray icon
+#' can be exported either as a blank value or as "0" (Incomplete). Blank values
+#' are recommended in a data export if the data will be re-imported into a
+#' REDCap project. Default is `FALSE`.
 #'
+#' @param http_response_encoding  The encoding value passed to
+#' [httr::content()].  Defaults to 'UTF-8'.
+#' @param locale a [readr::locale()] object to specify preferences like
+#' number, date, and time formats.  This object is passed to
+#' [readr::read_csv()].  Defaults to [readr::default_locale()].
 #' @param verbose A boolean value indicating if `message`s should be printed
 #' to the R console during the operation.  The verbose output might contain
 #' sensitive information (*e.g.* PHI), so turn this off if the output might
 #' be visible somewhere public. Optional.
-#' @param config_options  A list of options to pass to `POST` method in the
-#' `httr` package.  See the details below. Optional.
+#' @param config_options A list of options passed to [httr::POST()].
+#' See details at [httr::httr_options()]. Optional.
+#' @param handle_httr The value passed to the `handle` parameter of
+#' [httr::POST()].
+#' This is useful for only unconventional authentication approaches.  It
+#' should be `NULL` for most institutions.  Optional.
 #'
-#' @return Currently, a list is returned with the following elements:
-#' * `data`: An R [base::data.frame()] of the desired records and columns.
+#' @return
+#' Currently, a list is returned with the following elements:
+#' * `data`: A [tibble::tibble()] of the desired records and columns.
 #' * `success`: A boolean value indicating if the operation was apparently
 #' successful.
 #' * `status_code`: The
@@ -80,11 +93,6 @@
 #' empty string to save RAM.
 #'
 #' @details
-#' The full list of configuration options accepted by the `httr` package is
-#' viewable by executing [httr::httr_options()].  The `httr`
-#' package and documentation is available at
-#' https://cran.r-project.org/package=httr.
-#'
 #' If you do not pass in this export_data_access_groups value, it will
 #' default to `FALSE`. The following is from the API help page for version
 #' 5.2.3:
@@ -94,9 +102,11 @@
 #'
 #' As of REDCap 6.14.3, this field is not exported in the EAV API call.
 #'
-#' @author Will Beasley
+#' @author
+#' Will Beasley
 #'
-#' @references The official documentation can be found on the 'API Help Page'
+#' @references
+#' The official documentation can be found on the 'API Help Page'
 #' and 'API Examples' pages on the REDCap wiki (*i.e.*,
 #' https://community.projectredcap.org/articles/456/api-documentation.html and
 #' https://community.projectredcap.org/articles/462/api-examples.html).
@@ -108,20 +118,20 @@
 #' uri      <- "https://bbmc.ouhsc.edu/redcap/api/"
 #' token    <- "9A81268476645C4E5F03428B8AC3AA7B"
 #'
-#' #Return all records and all variables.
+#' # Return all records and all variables.
 #' ds <- REDCapR:::redcap_read_oneshot_eav(redcap_uri=uri, token=token)$data
 #'
-#' #Return only records with IDs of 1 and 3
+#' # Return only records with IDs of 1 and 3
 #' desired_records_v1 <- c(1, 3)
-#' ds_some_rows_v1    <- REDCapR::redcap_read_oneshot_eav(
+#' ds_some_rows_v1    <- REDCapR:::redcap_read_oneshot_eav(
 #'    redcap_uri = uri,
 #'    token      = token,
 #'    records    = desired_records_v1
 #' )$data
 #'
-#' #Return only the fields record_id, name_first, and age
+#' # Return only the fields record_id, name_first, and age
 #' desired_fields_v1 <- c("record_id", "name_first", "age")
-#' ds_some_fields_v1 <- redcap_read_oneshot_eav(
+#' ds_some_fields_v1 <- REDCapR:::redcap_read_oneshot_eav(
 #'    redcap_uri = uri,
 #'    token      = token,
 #'    fields     = desired_fields_v1
@@ -136,13 +146,9 @@ redcap_read_oneshot_eav <- function(
   redcap_uri,
   token,
   records                       = NULL,
-  records_collapsed             = "",
   fields                        = NULL,
-  fields_collapsed              = "",
   forms                         = NULL,
-  forms_collapsed               = "",
   events                        = NULL,
-  events_collapsed              = "",
   raw_or_label                  = "raw",
   raw_or_label_headers          = "raw",
   # placeholder: exportCheckboxLabel
@@ -152,23 +158,23 @@ redcap_read_oneshot_eav <- function(
   filter_logic                  = "",
   datetime_range_begin          = as.POSIXct(NA),
   datetime_range_end            = as.POSIXct(NA),
+  blank_for_gray_form_status    = FALSE,
 
   # placeholder: guess_type
   # placeholder: guess_max
+  http_response_encoding        = "UTF-8",
+  locale                        = readr::default_locale(),
   verbose                       = TRUE,
-  config_options                = NULL
+  config_options                = NULL,
+  handle_httr                   = NULL
 ) {
 
   checkmate::assert_character(redcap_uri                , any.missing=FALSE, len=1, pattern="^.{1,}$")
   checkmate::assert_character(token                     , any.missing=FALSE, len=1, pattern="^.{1,}$")
   checkmate::assert_atomic(records                      , any.missing=TRUE , min.len=0)
-  checkmate::assert_character(records_collapsed         , any.missing=TRUE , len=1, pattern="^.{0,}$", null.ok=TRUE )
   checkmate::assert_character(fields                    , any.missing=TRUE , min.len=1, pattern="^.{1,}$", null.ok=TRUE )
-  checkmate::assert_character(fields_collapsed          , any.missing=TRUE , len=1, pattern="^.{0,}$", null.ok=TRUE )
   checkmate::assert_character(forms                     , any.missing=TRUE , min.len=1, pattern="^.{1,}$", null.ok=TRUE )
-  checkmate::assert_character(forms_collapsed           , any.missing=TRUE , len=1, pattern="^.{0,}$", null.ok=TRUE )
   checkmate::assert_character(events                    , any.missing=TRUE , min.len=1, pattern="^.{1,}$", null.ok=TRUE )
-  checkmate::assert_character(events_collapsed          , any.missing=TRUE , len=1, pattern="^.{0,}$", null.ok=TRUE)
   checkmate::assert_character(raw_or_label              , any.missing=FALSE, len=1)
   checkmate::assert_subset(   raw_or_label              , c("raw", "label"))
   checkmate::assert_character(raw_or_label_headers      , any.missing=FALSE, len=1)
@@ -180,39 +186,40 @@ redcap_read_oneshot_eav <- function(
   checkmate::assert_character(filter_logic              , any.missing=FALSE, len=1, pattern="^.{0,}$")
   checkmate::assert_posixct(  datetime_range_begin      , any.missing=TRUE , len=1, null.ok=TRUE)
   checkmate::assert_posixct(  datetime_range_end        , any.missing=TRUE , len=1, null.ok=TRUE)
+  checkmate::assert_logical( blank_for_gray_form_status , any.missing=FALSE, len=1)
 
   # placeholder: checkmate::assert_logical(  guess_type                , any.missing=FALSE, len=1)
-  # placeholder: checkmate::assert_integerish(guess_max                , any.missing=FALSE, len=1, lower=1)
+  # placeholder: checkmate::assert_numeric(  guess_max                , any.missing=FALSE, len=1, lower=1)
+  checkmate::assert_character(http_response_encoding    , any.missing=FALSE,     len=1)
+  checkmate::assert_class(    locale, "locale"          , null.ok = FALSE)
   checkmate::assert_logical(  verbose                   , any.missing=FALSE, len=1, null.ok=TRUE)
   checkmate::assert_list(     config_options            , any.missing=TRUE ,        null.ok=TRUE)
 
-  validate_field_names(fields, stop_on_error = TRUE)
+  assert_field_names(fields)
 
-  token               <- sanitize_token(token)
-  records_collapsed   <- collapse_vector(records  , records_collapsed)
-  fields_collapsed    <- collapse_vector(fields   , fields_collapsed)
-  forms_collapsed     <- collapse_vector(forms    , forms_collapsed)
-  events_collapsed    <- collapse_vector(events   , events_collapsed)
+  token                     <- sanitize_token(token)
+  records_collapsed         <- collapse_vector(records)
+  fields_collapsed          <- collapse_vector(fields)
+  forms_collapsed           <- collapse_vector(forms)
+  events_collapsed          <- collapse_vector(events)
   export_data_access_groups <- ifelse(export_data_access_groups, "true", "false")
-  filter_logic        <- filter_logic_prepare(filter_logic)
-  datetime_range_begin<- dplyr::coalesce(strftime(datetime_range_begin, "%Y-%m-%d %H:%M:%S"), "")
-  datetime_range_end  <- dplyr::coalesce(strftime(datetime_range_end  , "%Y-%m-%d %H:%M:%S"), "")
-  verbose             <- verbose_prepare(verbose)
-
-  if (1L <= nchar(fields_collapsed) )
-    validate_field_names_collapsed(fields_collapsed, stop_on_error = TRUE)
+  filter_logic              <- filter_logic_prepare(filter_logic)
+  datetime_range_begin      <- dplyr::coalesce(strftime(datetime_range_begin, "%Y-%m-%d %H:%M:%S"), "")
+  datetime_range_end        <- dplyr::coalesce(strftime(datetime_range_end  , "%Y-%m-%d %H:%M:%S"), "")
+  verbose                   <- verbose_prepare(verbose)
 
   post_body <- list(
-    token                   = token,
-    content                 = "record",
-    format                  = "csv",
-    type                    = "eav",
-    rawOrLabel              = raw_or_label,
-    rawOrLabelHeaders       = raw_or_label_headers,
-    exportDataAccessGroups  = export_data_access_groups,
-    filterLogic             = filter_logic,
-    dateRangeBegin          = datetime_range_begin,
-    dateRangeEnd            = datetime_range_end
+    token                        = token,
+    content                      = "record",
+    format                       = "csv",
+    type                         = "eav",
+    rawOrLabel                   = raw_or_label,
+    rawOrLabelHeaders            = raw_or_label_headers,
+    exportDataAccessGroups       = export_data_access_groups,
+    filterLogic                  = filter_logic,
+    dateRangeBegin               = datetime_range_begin,
+    dateRangeEnd                 = datetime_range_end,
+    exportBlankForGrayFormStatus = blank_for_gray_form_status
     # record, fields, forms & events are specified below
   )
 
@@ -221,8 +228,14 @@ redcap_read_oneshot_eav <- function(
   if (0L < nchar(forms_collapsed  )) post_body$forms    <- forms_collapsed
   if (0L < nchar(events_collapsed )) post_body$events   <- events_collapsed
 
-  # This is the important line that communicates with the REDCap server.
-  kernel      <- kernel_api(redcap_uri, post_body, config_options)
+  # This is the important call that communicates with the REDCap server.
+  kernel <- kernel_api(
+    redcap_uri      = redcap_uri,
+    post_body       = post_body,
+    config_options  = config_options,
+    encoding        = http_response_encoding,
+    handle_httr     = handle_httr
+  )
 
   if (!kernel$success) {
     error_message     <- sprintf(
@@ -232,21 +245,30 @@ redcap_read_oneshot_eav <- function(
     stop(error_message)
   }
 
-
   ds_metadata <-
     REDCapR::redcap_metadata_read(
       redcap_uri,
       token,
-      forms_collapsed = forms_collapsed
+      forms           = forms,
+      handle_httr     = handle_httr,
+      verbose         = verbose
+    )$data
+
+  ds_variable <-
+    REDCapR::redcap_variables(
+      redcap_uri  = redcap_uri,
+      token       = token,
+      handle_httr = handle_httr,
+      verbose     = verbose
     )$data
-  ds_variable <- REDCapR::redcap_variables(redcap_uri, token)$data
 
   if (kernel$success) {
-    try (
+    try(
       {
         ds_eav <-
           readr::read_csv(
             file            = I(kernel$raw_text),
+            locale          = locale,
             show_col_types  = FALSE
           )
 
@@ -256,15 +278,15 @@ redcap_read_oneshot_eav <- function(
 
         ds_metadata_expanded <-
           ds_metadata %>%
-          dplyr::select(.data$field_name, .data$select_choices_or_calculations, .data$field_type) %>%
+          dplyr::select("field_name", "select_choices_or_calculations", "field_type") %>%
           dplyr::mutate(
             is_checkbox   = (.data$field_type == "checkbox"),
             ids           = dplyr::if_else(.data$is_checkbox, .data$select_choices_or_calculations, "1"),
-            ids           = gsub("(\\d+),.+?(\\||$)", "\\1", .data$ids),
+            ids           = gsub("(\\w+),.+?(\\||$)", "\\1", .data$ids),
             ids           = strsplit(.data$ids, " ")
           ) %>%
-          dplyr::select(-.data$select_choices_or_calculations, -.data$field_type) %>%
-          tidyr::unnest(.data$ids) %>%
+          dplyr::select(-"select_choices_or_calculations", -"field_type") %>%
+          tidyr::unnest("ids") %>%
           dplyr::transmute(
             .data$is_checkbox,
             field_name          = dplyr::if_else(.data$is_checkbox, paste0(.data$field_name, "___", .data$ids), .data$field_name)
@@ -286,10 +308,10 @@ redcap_read_oneshot_eav <- function(
 
         variables_to_keep <-
           ds_metadata_expanded %>%
-          dplyr::select(.data$field_name) %>%
+          dplyr::select("field_name") %>%
           dplyr::union(
             ds_variable %>%
-              dplyr::select(field_name = .data$export_field_name) %>%
+              dplyr::select(field_name = "export_field_name") %>%
               dplyr::filter(grepl("^\\w+?_complete$", .data$field_name))
           ) %>%
           dplyr::pull(.data$field_name) #%>% rev()
@@ -298,7 +320,7 @@ redcap_read_oneshot_eav <- function(
           ds_eav %>%
           dplyr::left_join(
             ds_metadata %>%
-              dplyr::select(.data$field_name, .data$field_type),
+              dplyr::select("field_name", "field_type"),
             by = "field_name"
           ) %>%
           dplyr::mutate(
@@ -306,14 +328,19 @@ redcap_read_oneshot_eav <- function(
           ) %>%
           dplyr::full_join(ds_possible_checkbox_rows, by=c("record", "field_name", "field_type", "event_id")) %>%
           dplyr::mutate(
-            value      = dplyr::if_else(!is.na(.data$field_type) & (.data$field_type == "checkbox"), as.character(!is.na(.data$value)), .data$value)
+            value =
+              dplyr::if_else(
+                !is.na(.data$field_type) & (.data$field_type == "checkbox"),
+                as.character(!is.na(.data$value)),
+                as.character(.data$value)
+              )
           )
 
         . <- NULL # For the sake of avoiding an R CMD check note.
         ds <-
           ds_eav_2 %>%
-          dplyr::select(-.data$field_type) %>%
-          # dplyr::select(-.data$redcap_repeat_instance) %>%        # TODO: need a good fix for repeats
+          dplyr::select(-"field_type") %>%
+          # dplyr::select(-"redcap_repeat_instance") %>%        # TODO: need a good fix for repeats
           # tidyr::drop_na(event_id) %>%                            # TODO: need a good fix for repeats
           tidyr::spread(key = .data$field_name, value = .data$value) %>%
           dplyr::select(.data = ., !!intersect(variables_to_keep, colnames(.)))
@@ -322,8 +349,8 @@ redcap_read_oneshot_eav <- function(
           ds %>%
           dplyr::mutate_if(is.character, ~type.convert(., as.is = FALSE)) %>%
           dplyr::mutate_if(is.factor   , as.character)
-      }, #Convert the raw text to a dataset.
-      silent = TRUE #Don't print the warning in the try block.  Print it below, where it's under the control of the caller.
+      },
+      silent = TRUE # Don't print the warning in the try block.  Print it below, where it's under the control of the caller.
     )
 
     if (ifelse(exists("ds_2"), inherits(ds_2, "data.frame"), FALSE)) {
@@ -338,8 +365,8 @@ redcap_read_oneshot_eav <- function(
       kernel$raw_text   <- "" # If an operation is successful, the `raw_text` is no longer returned to save RAM.  The content is not really necessary with httr's status message exposed.
     } else {
       # nocov start
-      kernel$success    <- FALSE #Override the 'success' determination from the http status code.
-      ds_2              <- tibble::tibble() #Return an empty data.frame
+      kernel$success    <- FALSE # Override the 'success' determination from the http status code.
+      ds_2              <- tibble::tibble() # Return an empty data.frame
       outcome_message   <- sprintf(
         "The REDCap read failed.  The http status code was %s.  The 'raw_text' returned was '%s'.",
         kernel$status_code,
@@ -349,15 +376,16 @@ redcap_read_oneshot_eav <- function(
     }
   } else {
     # nocov start
-    ds_2            <- tibble::tibble() #Return an empty data.frame
-    outcome_message <- if (any(grepl(kernel$regex_empty, kernel$raw_text))) {
-      "The REDCapR read/export operation was not successful.  The returned dataset was empty."
-    } else {
-      sprintf(
-        "The REDCapR read/export operation was not successful.  The error message was:\n%s",
-        kernel$raw_text
-      )
-    }
+    ds_2            <- tibble::tibble() # Return an empty data.frame
+    outcome_message <-
+      if (any(grepl(kernel$regex_empty, kernel$raw_text))) {
+        "The REDCapR read/export operation was not successful.  The returned dataset was empty."
+      } else {
+        sprintf(
+          "The REDCapR read/export operation was not successful.  The error message was:\n%s",
+          kernel$raw_text
+        )
+      }
     # nocov end
   }
 
@@ -365,17 +393,17 @@ redcap_read_oneshot_eav <- function(
     message(outcome_message)
 
   list(
-    data               = ds_2,
-    success            = kernel$success,
-    status_code        = kernel$status_code,
-    outcome_message    = outcome_message,
-    records_collapsed  = records_collapsed,
-    fields_collapsed   = fields_collapsed,
-    filter_logic       = filter_logic,
-    datetime_range_begin= datetime_range_begin,
-    datetime_range_end  = datetime_range_end,
-    events_collapsed   = events_collapsed,
-    elapsed_seconds    = kernel$elapsed_seconds,
-    raw_text           = kernel$raw_text
+    data                  = ds_2,
+    success               = kernel$success,
+    status_code           = kernel$status_code,
+    outcome_message       = outcome_message,
+    records_collapsed     = records_collapsed,
+    fields_collapsed      = fields_collapsed,
+    filter_logic          = filter_logic,
+    datetime_range_begin  = datetime_range_begin,
+    datetime_range_end    = datetime_range_end,
+    events_collapsed      = events_collapsed,
+    elapsed_seconds       = kernel$elapsed_seconds,
+    raw_text              = kernel$raw_text
   )
 }
