@@ -1,19 +1,25 @@
-#' @title Write/Import records to a REDCap project
+#' @title
+#' Write/Import records to a REDCap project
 #'
-#' @description This function uses REDCap's API to select and return data.
+#' @description
+#' This function uses REDCap's API to select and return data.
 #'
-#' @param ds The [base::data.frame()] to be imported into the REDCap project.
+#' @param ds The [base::data.frame()] or [tibble::tibble()]
+#' to be imported into the REDCap project.
+#' Required.
+#' @param redcap_uri The
+#' [uri](https://en.wikipedia.org/wiki/Uniform_Resource_Identifier)/url
+#' of the REDCap server
+#' typically formatted as "https://server.org/apps/redcap/api/".
 #' Required.
-#' @param redcap_uri The URI (uniform resource identifier) of the REDCap
-#' project.  Required.
 #' @param token The user-specific string that serves as the password for a
 #' project.  Required.
 #' @param overwrite_with_blanks A boolean value indicating if
-#' blank/`NA` values in the R [base::data.frame]
+#' blank/`NA` values in the R data frame
 #' will overwrite data on the server.
 #' This is the default behavior for REDCapR,
 #' which essentially deletes the cell's value
-#' If `FALSE`, blank/`NA` values in the [base::data.frame]
+#' If `FALSE`, blank/`NA` values in the data frame
 #' will be ignored.  Optional.
 #' @param convert_logical_to_integer If `TRUE`, all [base::logical] columns
 #' in `ds` are cast to an integer before uploading to REDCap.
@@ -23,10 +29,15 @@
 #' to the R console during the operation.  The verbose output might contain
 #' sensitive information (*e.g.* PHI), so turn this off if the output might
 #' be visible somewhere public. Optional.
-#' @param config_options  A list of options to pass to [httr::POST()] method
-#' in the 'httr' package.  See the details in [redcap_read_oneshot()] Optional.
+#' @param config_options A list of options passed to [httr::POST()].
+#' See details at [httr::httr_options()]. Optional.
+#' @param handle_httr The value passed to the `handle` parameter of
+#' [httr::POST()].
+#' This is useful for only unconventional authentication approaches.  It
+#' should be `NULL` for most institutions.  Optional.
 #'
-#' @return Currently, a list is returned with the following elements:
+#' @return
+#' Currently, a list is returned with the following elements:
 #' * `success`: A boolean value indicating if the operation was apparently
 #' successful.
 #' * `status_code`: The
@@ -46,9 +57,11 @@
 #' REDCap's supported variables.  See [validate_for_write()] for a helper
 #' function that checks for some common important conflicts.
 #'
-#' @author Will Beasley
+#' @author
+#' Will Beasley
 #'
-#' @references The official documentation can be found on the 'API Help Page'
+#' @references
+#' The official documentation can be found on the 'API Help Page'
 #' and 'API Examples' pages on the REDCap wiki (*i.e.*,
 #' https://community.projectredcap.org/articles/456/api-documentation.html and
 #' https://community.projectredcap.org/articles/462/api-examples.html).
@@ -56,8 +69,8 @@
 #' administrator to send you the static material.
 #'
 #' @examples
-#' \dontrun{
-#' #Define some constants
+#' if (FALSE) {
+#' # Define some constants
 #' uri            <- "https://bbmc.ouhsc.edu/redcap/api/"
 #' token          <- "D70F9ACD1EDD6F151C6EA78683944E98"
 #'
@@ -70,7 +83,7 @@
 #' ds1$telephone  <- paste0("(405) 321-000", seq_len(nrow(ds1)))
 #'
 #' ds1 <- ds1[1:3, ]
-#' ds1$age        <- NULL; ds1$bmi <- NULL #Drop the calculated fields before writing.
+#' ds1$age        <- NULL; ds1$bmi <- NULL # Drop the calculated fields before writing.
 #' result_write   <- REDCapR::redcap_write_oneshot(ds=ds1, redcap_uri=uri, token=token)
 #'
 #' # Read the dataset for the second time.
@@ -86,6 +99,7 @@
 #' result_write$raw_text
 #' }
 
+#' @importFrom magrittr %>%
 #' @export
 redcap_write_oneshot <- function(
   ds,
@@ -93,11 +107,15 @@ redcap_write_oneshot <- function(
   token,
   overwrite_with_blanks         = TRUE,
   convert_logical_to_integer    = FALSE,
-  verbose         = TRUE,
-  config_options  = NULL
+  verbose                       = TRUE,
+  config_options                = NULL,
+  handle_httr                   = NULL
 ) {
 
-  csv_elements <- NULL #This prevents the R CHECK NOTE: 'No visible binding for global variable Note in R CMD check';  Also see  if( getRversion() >= "2.15.1" )    utils::globalVariables(names=c("csv_elements")) #https://stackoverflow.com/questions/8096313/no-visible-binding-for-global-variable-note-in-r-cmd-check; https://stackoverflow.com/questions/9439256/how-can-i-handle-r-cmd-check-no-visible-binding-for-global-variable-notes-when
+  # This prevents the R CHECK NOTE: 'No visible binding for global variable Note in R CMD check';
+  # Also see  if( getRversion() >= "2.15.1" )    utils::globalVariables(names=c("csv_elements"))
+  # https://stackoverflow.com/questions/8096313/; https://stackoverflow.com/questions/9439256
+  csv_elements <- NULL
 
   checkmate::assert_character(redcap_uri, any.missing=FALSE, len=1, pattern="^.{1,}$")
   checkmate::assert_character(token     , any.missing=FALSE, len=1, pattern="^.{1,}$")
@@ -129,8 +147,8 @@ redcap_write_oneshot <- function(
     format    = "csv",
     type      = "flat",
 
-    #These next values separate the import from the export API call
-    #overwriteBehavior:
+    # These next values separate the import from the export API call
+    # overwriteBehavior:
     #  *normal* - blank/empty values will be ignored [default];
     #  *overwrite* - blank/empty values are valid and will overwrite data
 
@@ -140,8 +158,14 @@ redcap_write_oneshot <- function(
     returnFormat        = "csv"
   )
 
-  # This is the important line that communicates with the REDCap server.
-  kernel <- kernel_api(redcap_uri, post_body, config_options)
+  # This is the important call that communicates with the REDCap server.
+  kernel <-
+    kernel_api(
+      redcap_uri      = redcap_uri,
+      post_body       = post_body,
+      config_options  = config_options,
+      handle_httr     = handle_httr
+    )
 
   if (kernel$success) {
     elements               <- unlist(strsplit(kernel$raw_text, split="\\n"))
@@ -153,9 +177,9 @@ redcap_write_oneshot <- function(
       kernel$elapsed_seconds
     )
 
-    #If an operation is successful, the `raw_text` is no longer returned to save RAM.  The content is not really necessary with httr's status message exposed.
+    # If an operation is successful, the `raw_text` is no longer returned to save RAM.  The content is not really necessary with httr's status message exposed.
     kernel$raw_text <- ""
-  } else { #If the returned content wasn't recognized as valid IDs, then
+  } else { # If the returned content wasn't recognized as valid IDs, then
     affected_ids           <- character(0) # Return an empty array
     records_affected_count <- NA_integer_
     outcome_message        <- sprintf(
