@@ -1,4 +1,4 @@
-## ----set_options, echo=FALSE, results='hide'----------------------------------
+## ----set_options--------------------------------------------------------------
 report_render_start_time <- Sys.time()
 
 library(knitr)
@@ -6,35 +6,11 @@ library(magrittr)
 requireNamespace("kableExtra")
 
 opts_chunk$set(
-  comment = NA,
+  collapse = TRUE,
+  comment = "#>",
   tidy    = FALSE
 )
 
-knit_print.data.frame = function(x, ...) {
-  # See https://cran.r-project.org/package=knitr/vignettes/knit_print.html
-  # res = paste(c("", "", kable(x)), collapse = "\n")
-  # asis_output(res)
-  x %>%
-    # dplyr::mutate_if(
-    #   is.character,
-    #   function( s ) gsub("\\n", "<br/>", s)
-    # ) %>%
-    kable(
-      col.names = gsub("_", " ", colnames(.)),
-      format    = "html"
-    ) %>%
-    kableExtra::kable_styling(
-      bootstrap_options = c("striped", "hover", "condensed", "responsive"),
-      full_width        = FALSE
-    ) %>%
-    c("", "", .) %>%
-    paste(collapse = "\n") %>%
-    asis_output()
-
-}
-# options(markdown.HTML.header = system.file("misc/vignette.css", package = "REDCapR"))
-# options(width=120) #So the output is 50% wider than the default.
-
 ## ----project_values-----------------------------------------------------------
 library(REDCapR) # Load the package into the current R session.
 uri   <- "https://bbmc.ouhsc.edu/redcap/api/"
@@ -45,44 +21,24 @@ token <- "9A81268476645C4E5F03428B8AC3AA7B" # `UnitTestPhiFree` user and simple
 ds_all_rows_all_fields <- redcap_read(redcap_uri = uri, token = token)$data
 ds_all_rows_all_fields # Inspect the returned dataset
 
-## ----read_row_subset, results='hold'------------------------------------------
+## ----read_row_subset----------------------------------------------------------
 # Return only records with IDs of 1 and 3
-desired_records_v1 <- c(1, 3)
+desired_records <- c(1, 3)
 ds_some_rows_v1 <- redcap_read(
   redcap_uri = uri,
   token      = token,
-  records    = desired_records_v1
+  records    = desired_records
 )$data
 
-# Return only records with IDs of 1 and 3 (alternate way)
-desired_records_v2 <- "1, 3"
-ds_some_rows_v2 <- redcap_read(
-  redcap_uri        = uri,
-  token             = token,
-  records_collapsed = desired_records_v2
-)$data
-
-ds_some_rows_v2 # Inspect the returned dataset
-
 ## ----read_field_subset--------------------------------------------------------
-#Return only the fields record_id, name_first, and age
-desired_fields_v1 <- c("record_id", "name_first", "age")
-ds_some_fields_v1 <- redcap_read(
+# Return only the fields record_id, name_first, and age
+desired_fields <- c("record_id", "name_first", "age")
+ds_some_fields <- redcap_read(
   redcap_uri = uri,
   token      = token,
-  fields     = desired_fields_v1
+  fields     = desired_fields
 )$data
 
-#Return only the fields record_id, name_first, and age (alternate way)
-desired_fields_v2 <- "record_id, name_first, age"
-ds_some_fields_v2 <- redcap_read(
-  redcap_uri       = uri,
-  token            = token,
-  fields_collapsed = desired_fields_v2
-)$data
-
-ds_some_fields_v2 #Inspect the returned dataset
-
 ## ----read_record_field_subset-------------------------------------------------
 ######
 ## Step 1: First call to REDCap
@@ -114,12 +70,12 @@ ds_some_rows_v3 <- redcap_read(
 
 ds_some_rows_v3 #Examine the results.
 
-## ----read_not_just_dataframe--------------------------------------------------
+## -----------------------------------------------------------------------------
 #Return only the fields record_id, name_first, and age
 all_information <- redcap_read(
   redcap_uri = uri,
   token      = token,
-  fields     = desired_fields_v1
+  fields     = desired_fields
 )
 all_information #Inspect the additional information
 
@@ -131,5 +87,5 @@ if (requireNamespace("sessioninfo", quietly = TRUE)) {
 }
 
 ## ----session-duration, echo=FALSE---------------------------------------------
-report_render_duration_in_seconds <- round(as.numeric(difftime(Sys.time(), report_render_start_time, units = "secs")))
+report_render_duration <- round(as.numeric(difftime(Sys.time(), report_render_start_time, units = "secs")))
 
