@@ -4,7 +4,7 @@ credential  <- retrieve_credential_testing(1490L)
 
 # Read in the dictionary in R's memory from a csv file.
 path_in <- system.file(
-  "test-data/project-simple/simple-metadata.csv",
+  "test-data/project-simple/metadata.csv",
   package = "REDCapR"
 )
 dictionary_to_write <-
@@ -33,7 +33,7 @@ test_that("Metadata Write", {
   testthat::expect_equal(returned_object$field_count, 16L)
   testthat::expect_equal(returned_object$raw_text, "")
 })
-test_that("Metadata Writen -Error", {
+test_that("Metadata Write -Error", {
   testthat::skip_on_cran()
 
   testthat::expect_message(
