@@ -1,27 +1,17 @@
 library(testthat)
 
-test_that("both NULL", {
+test_that("NULL", {
   elements  <- NULL
-  collapsed <- NULL
   expected  <- ""
 
-  observed  <- REDCapR:::collapse_vector(elements, collapsed)
+  observed  <- REDCapR:::collapse_vector(elements)
   expect_equal(observed, expected)
 })
-test_that("collapsed NULL", {
+test_that("specified", {
   elements  <- letters
-  collapsed <- NULL
   expected  <- "a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y,z"
 
-  observed  <- REDCapR:::collapse_vector(elements, collapsed)
-  expect_equal(observed, expected)
-})
-test_that("elements NULL", {
-  elements  <- NULL
-  collapsed <- "4,5,6"
-  expected  <- collapsed
-
-  observed  <- REDCapR:::collapse_vector(elements, collapsed)
+  observed  <- REDCapR:::collapse_vector(elements)
   expect_equal(observed, expected)
 })
 
